@@ -36,6 +36,7 @@ import inspect
 import itertools
 import json
 import uuid
+import warnings
 
 from oslo_utils import encodeutils
 from oslo_utils import importutils
@@ -52,12 +53,12 @@ _nasty_type_tests = [inspect.ismodule, inspect.isclass, inspect.ismethod,
                      inspect.iscode, inspect.isbuiltin, inspect.isroutine,
                      inspect.isabstract]
 
-_simple_types = (six.string_types + six.integer_types
+_simple_types = ((six.text_type,) + six.integer_types
                  + (type(None), bool, float))
 
 
 def to_primitive(value, convert_instances=False, convert_datetime=True,
-                 level=0, max_depth=3):
+                 level=0, max_depth=3, encoding='utf-8'):
     """Convert a complex object into primitives.
 
     Handy for JSON serialization. We can optionally handle instances,
@@ -92,6 +93,11 @@ def to_primitive(value, convert_instances=False, convert_datetime=True,
     if isinstance(value, _simple_types):
         return value
 
+    if isinstance(value, six.binary_type):
+        if six.PY3:
+            value = value.decode(encoding=encoding)
+        return value
+
     # It's not clear why xmlrpclib created their own DateTime type, but
     # for our purposes, make it a datetime type which is explicitly
     # handled
@@ -107,7 +113,7 @@ def to_primitive(value, convert_instances=False, convert_datetime=True,
     if isinstance(value, uuid.UUID):
         return six.text_type(value)
 
-    if netaddr and isinstance(value, netaddr.IPAddress):
+    if netaddr and isinstance(value, (netaddr.IPAddress, netaddr.IPNetwork)):
         return six.text_type(value)
 
     if ipaddress and isinstance(value,
@@ -141,10 +147,11 @@ def to_primitive(value, convert_instances=False, convert_datetime=True,
                                       convert_instances=convert_instances,
                                       convert_datetime=convert_datetime,
                                       level=level,
-                                      max_depth=max_depth)
+                                      max_depth=max_depth,
+                                      encoding=encoding)
         if isinstance(value, dict):
             return {recursive(k): recursive(v)
-                    for k, v in six.iteritems(value)}
+                    for k, v in value.items()}
         elif hasattr(value, 'iteritems'):
             return recursive(dict(value.iteritems()), level=level + 1)
         # Python 3 does not have iteritems
@@ -161,6 +168,9 @@ def to_primitive(value, convert_instances=False, convert_datetime=True,
         # __iter__ defined but it isn't callable as list().
         return six.text_type(value)
 
+    # TODO(gcb) raise ValueError in version 3.0
+    warnings.warn("Cannot convert %r to primitive, will raise ValueError "
+                  "instead of warning in version 3.0" % (value,))
     return value
 
 
